SystemOrganization addCategory: #BackgammonModel!

TestCase subclass: #CheckerTestCase
	instanceVariableNames: 'checkerObserver checkerOnBar checkerOnPoint mockCheckerView barLocation pointLocation movableChecker'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!CheckerTestCase methodsFor: 'testing' stamp: 'jChizmar 7/17/2015 18:54'!
setUp
	barLocation := BarLocation new .
	pointLocation := PointLocation new number:1.
	checkerOnPoint := Checker new location: pointLocation color:  (Color red).
	checkerOnBar := Checker new location: barLocation color:  (Color red).
	
	movableChecker := Checker new location: barLocation color:  (Color blue).
	movableChecker initialize .
	mockCheckerView := MockView new.
	movableChecker addObserver: mockCheckerView.! !

!CheckerTestCase methodsFor: 'testing' stamp: 'AhmedSoliman 7/13/2015 14:59'!
testInitialize
	"Create a Checker"
	
	Transcript show:(checkerOnPoint printString).
	self assert: (checkerOnPoint isNotNil ).
	self assert:(checkerOnPoint color = Color red).

! !

!CheckerTestCase methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/15/2015 23:49'!
testObserverWithCommand
	"test moving the checker from bar to point"
	movableChecker moveChecker: pointLocation ! !

!CheckerTestCase methodsFor: 'testing' stamp: 'jChizmar 7/15/2015 21:30'!
testPrinting
	"Tests for printing out Checker"
	
	Transcript show:(checkerOnBar printString).
	self assert:(checkerOnBar printString = ('Checker Class -', (String cr) , 'Color:Color red', (String cr) , 'Location:BarLocation Class') ).
	
	self assert:(((checkerOnPoint checkerLocation) printString) = ('PointLocation Class -', (String cr), 'PointNumber:', '1')).

	! !

TestCase subclass: #DiceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!DiceTest methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/21/2015 11:34'!
testDice
	"comment stating purpose of message"
	|myRoll myCollection i myDice|
	myDice := Dice initialize.
	myRoll := myDice roll.
	self assert:(myRoll >=1 & myRoll <=6).
	myCollection := OrderedCollection new.
	i := 0.
	[i<100] whileTrue:[
		myRoll := myDice roll.
		self assert:(myRoll >=1 & myRoll <=6).
		myCollection add: myRoll.
		i  :=i+1.
	].
 
	! !

TestCase subclass: #MainBoardTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!MainBoardTest methodsFor: 'testing' stamp: 'jChizmar 7/23/2015 20:03'!
testBearingOff
	"tests Around determing if a player can Bear off"
		|myMainBoard|
	
	myMainBoard := MainBoard initialize.
	"By Default the board in this test is at the beginning of the game"
	self assert:((myMainBoard canBearOff: 1) = false).
	self assert:((myMainBoard canBearOff: 2) = false).
	
	"setup board to be at the end of the game"
	myMainBoard player1Checkers:OrderedCollection new.
	myMainBoard player2Checkers:OrderedCollection new.
	
	MainBoard initializeBearingOffPlayer1Checkers:myMainBoard checkerColor:Color red.
	MainBoard initializeBearingOffPlayer2Checkers:myMainBoard checkerColor:Color black.
	self assert:(myMainBoard canBearOff: 1 = true).
	self assert:(myMainBoard canBearOff: 2 = true).
	
! !

!MainBoardTest methodsFor: 'testing' stamp: 'jChizmar 7/20/2015 22:39'!
testCheckersAtLocation
	"testing finding the checkers for a given location"
	
	|mainBoardToTest locationList i|
	mainBoardToTest := MainBoard initialize.
	
	"Test with just the Initizlized Board Settings"
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:2).
	self assert:( locationList size = 0).
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:24).
	self assert:( locationList size = 2).
	
	"Test with Different Combos of Checkers"
	
	"Multiple Player 1 checkers on the same point one on a different point"
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:5).
	self assert:( locationList size = 0).
	i:=0.
	[i<5  ] whileTrue:[
		mainBoardToTest player1Checkers addLast: (Checker new location: (mainBoardToTest points at: 5) color: Color red).
		i:=i+1.
	].
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:5).
	self assert:( locationList size = 5).
	
	"Multiple Player 2 checkers on the same point one on a different point"
	i:=0.
	[i<5  ] whileTrue:[
		mainBoardToTest player2Checkers addLast: (Checker new location: (mainBoardToTest points at: 6) color: Color red).
		i:=i+1.
	].
locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:12).
	self assert:( locationList size = 5).
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:19).
	self assert:( locationList size = 5).
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:17).
	self assert:( locationList size = 3).
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:1).
	self assert:( locationList size = 2).
	
	"Check for Player 1 checkers on the Bar"
	
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:5).
	(locationList at:1) moveChecker: mainBoardToTest bar.
	locationList := mainBoardToTest checkersOnLocation:(mainBoardToTest bar).
	self assert:( locationList size = 1 & (locationList at:1) color = Color red).
	
	
	
	"Check for Player 1 & 2 checkers on the Bar"
	locationList := mainBoardToTest checkersOnLocation: (mainBoardToTest points at:6).
	(locationList at:1) moveChecker: mainBoardToTest bar.
	locationList := mainBoardToTest checkersOnLocation:(mainBoardToTest bar).
	self assert:( locationList size = 2).
	! !

!MainBoardTest methodsFor: 'testing' stamp: 'jChizmar 7/20/2015 22:38'!
testMainBoardInit
	"comment stating purpose of message"
	
	|myMainBoard locationList|
	
	myMainBoard := MainBoard initialize.
	self assert: ((myMainBoard player1Checkers) size = 15).
	self assert: ((myMainBoard player2Checkers) size = 15).
	self assert: (((myMainBoard player1Checkers at:1) color = Color red) ).
	self assert: (((myMainBoard player2Checkers at:1) color = Color black) ).
	
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:12).
	self assert:( locationList size = 5).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:19).
	self assert:( locationList size = 5).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:17).
	self assert:( locationList size = 3).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:1).
	self assert:( locationList size = 2).
	
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:2).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:3).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:4).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:5).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:7).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:9).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:10).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:11).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:14).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:15).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:20).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:21).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:22).
	self assert:( locationList size = 0).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:23).
	self assert:( locationList size = 0).

	
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:13).
	self assert:( locationList size = 5).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:6).
	self assert:( locationList size = 5).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:8).
	self assert:( locationList size = 3).
	locationList := myMainBoard checkersOnLocation: (myMainBoard points at:24).
	self assert:( locationList size = 2).
	
	self assert: (myMainBoard bar = BarLocation new).
	self assert: ((myMainBoard points) size = 24).
	self assert: ((myMainBoard points at: 4) pointNumber = 4).
	self assert: ((myMainBoard points at: 1) pointNumber = 1).
	self assert: ((myMainBoard player1) nameOfPlayer = 'Jen').
	self assert: ((myMainBoard player2) nameOfPlayer = 'Karen').! !

!MainBoardTest methodsFor: 'testing' stamp: 'jChizmar 7/24/2015 07:18'!
testObserverSimple
	"comment stating purpose of message"

	|myObserver myChecker barLocation|
	barLocation := BarLocation new.
	myChecker := Checker new location: barLocation color:  (Color red).
	
	myObserver := MockView new.
	myChecker addObserver: myObserver.
	
	myChecker moveChecker: (PointLocation new number:3).
	self assert:((myObserver observable) checkerLocation pointNumber = 3).! !

!MainBoardTest methodsFor: 'testing' stamp: 'jChizmar 7/24/2015 19:23'!
testObserverWithCommand
	"comment stating purpose of message"

	|myMoveCommand myObserver myMainBoard|
	
	myMainBoard := MainBoard initialize.
	myMainBoard dice1:(DiceMock initialize:1).
	myMainBoard dice2:(DiceMock initialize:2).
	myMainBoard dice1 roll.
	myMainBoard dice2 roll.
	
	myObserver := MockView new .
	(myMainBoard player1Checkers at: 1) addObserver: myObserver.
	myMainBoard itsMyTurn:(myMainBoard player1).
	
	myMoveCommand := MoveCheckerCommand new checker: (myMainBoard player1Checkers at: 1) location: (myMainBoard points at:2).
	myMoveCommand execute.
	self assert:((myMainBoard player1Checkers at: 1) checkerLocation pointNumber = 2).
	self assert:(myObserver observable checkerLocation pointNumber = 2).! !

TestCase subclass: #MoveCheckerCommandTestCase
	instanceVariableNames: 'barLocation pointLocation checkerOnPoint checkerOnBar testPlayer1Off testPlayer2Off testBoard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!MoveCheckerCommandTestCase methodsFor: 'initialize' stamp: 'jChizmar 7/21/2015 19:58'!
setUp
	"Setup the Objects for Testing"
	testBoard := MainBoard initialize.
	testBoard dice1 roll.
	testBoard dice2 roll.
	barLocation := testBoard bar.
	checkerOnPoint := Checker new location: (testBoard points at:1) color:  (Color red) .
	checkerOnBar := Checker new location: barLocation color:  (Color red).
! !

!MoveCheckerCommandTestCase methodsFor: 'testing' stamp: 'jChizmar 7/25/2015 00:08'!
testBearingOff
	"Test that a player can bear off and the OffLocation updates"
	
	|myMainBoard checkerInTest moveCheckerCommand |
	
	myMainBoard := MainBoard initialize.
	"By Default the board in this test is at the beginning of the game - test that the logic does not allow you to Bear off"
	self assert:((myMainBoard canBearOff: 1) = false).
	self assert:((myMainBoard canBearOff: 2) = false).
	
	
	myMainBoard dice1: (DiceMock initialize:1).
	myMainBoard dice2: (DiceMock initialize:2).
	myMainBoard dice1 roll.
	myMainBoard dice2 roll.
	
	checkerInTest := myMainBoard player1Checkers at:1.
	myMainBoard itsMyTurn: (myMainBoard player1).
 	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (myMainBoard player1Off).
	self should:  [moveCheckerCommand execute.]
			 raise: InvalidMoveException. 
	
	"setup board to be at the end of the game"
	myMainBoard player1Checkers:OrderedCollection new.
	myMainBoard player2Checkers:OrderedCollection new.
	
	MainBoard initializeBearingOffPlayer1Checkers:myMainBoard checkerColor:Color red.
	MainBoard initializeBearingOffPlayer2Checkers:myMainBoard checkerColor:Color black.
	
	checkerInTest := myMainBoard player1Checkers at:1.
	myMainBoard dice1: (DiceMock initialize:6).
	myMainBoard dice2: (DiceMock initialize:2).
	myMainBoard dice1 roll.
	myMainBoard dice2 roll.

	self assert:((myMainBoard canBearOff: 1) = true).
	self assert:((myMainBoard canBearOff: 2) = true).
	self assert:((myMainBoard player1Off numCheckersOnHome)  = 0).
	self assert:((myMainBoard player2Off numCheckersOnHome)  = 0).

	"In this scenario, there are still checkers left of the dice to Bear Off"
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (myMainBoard player1Off).

	moveCheckerCommand execute.
	self assert:(checkerInTest checkerLocation = myMainBoard player1Off).
	self assert:(checkerInTest checkerLocation numCheckersOnHome = 1).
	
	checkerInTest := myMainBoard player1Checkers at:2.
	myMainBoard dice1: (DiceMock initialize:5 ).
	myMainBoard dice2: (DiceMock initialize:2).
	myMainBoard dice1 roll.
	myMainBoard dice2 roll.

	self assert:((myMainBoard canBearOff: 1) = true).
	self assert:((myMainBoard canBearOff: 2) = true).
	self assert:((myMainBoard player1Off numCheckersOnHome)  = 1).
	self assert:((myMainBoard player2Off numCheckersOnHome)  = 0).

	"In this scenario, there are zero checkers to the left of the selected one - thus you can move on dice >= location"
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (myMainBoard player1Off).
	moveCheckerCommand execute.
	self assert:(checkerInTest checkerLocation = myMainBoard player1Off).
	self assert:(checkerInTest checkerLocation numCheckersOnHome = 2).! !

!MoveCheckerCommandTestCase methodsFor: 'testing' stamp: 'jChizmar 7/23/2015 16:15'!
testDoubles
	"comment stating purpose of message"
"Player 1 rolls"
	| checkerInTest moveCheckerCommand |
	testBoard dice1: (DiceMock initialize:1).
	testBoard dice2: (DiceMock initialize:1).
	testBoard dice1 roll.
	testBoard dice2 roll.
	
	checkerInTest := testBoard player1Checkers at:1.
	"Clear out Player 2 checkers - for this test"
	testBoard player2Checkers: OrderedCollection new.
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (testBoard points at:2).
	moveCheckerCommand execute. "Execute First,  Dice, First Move - 1"
	self assert: (checkerInTest checkerLocation pointNumber = 2).
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (testBoard points at:3).
	moveCheckerCommand execute. "Execute First Dice, Second Move - 1"
	self assert: (checkerInTest checkerLocation pointNumber = 3).
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (testBoard points at:4).
	moveCheckerCommand execute. "Execute Second Dice, First Move - 1"
	self assert: (checkerInTest checkerLocation pointNumber = 4).
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (testBoard points at:5).
	moveCheckerCommand execute. "Execute Second Dice, Second Move - 1"
	self assert: (checkerInTest checkerLocation pointNumber = 5).
	"Too Many Moves "
	moveCheckerCommand := (MoveCheckerCommand new)  checker:(checkerInTest)  location: (testBoard points at:8).
		self should:  [
			moveCheckerCommand execute.
		]
			 raise: InvalidMoveException.! !

!MoveCheckerCommandTestCase methodsFor: 'initialize' stamp: 'jChizmar 7/21/2015 20:00'!
testInitializeAndExecute
	"Tests the Initialization and setup of the MoveCheckerCommand"
	
	|moveCheckerCommand|
	moveCheckerCommand := (MoveCheckerCommand new) checker:checkerOnPoint  location: barLocation.
		self should:  [moveCheckerCommand execute.]
			 raise: InvalidMoveException.
	"self assert:((checkerOnPoint checkerLocation) = barLocation)."

! !

!MoveCheckerCommandTestCase methodsFor: 'testing' stamp: 'jChizmar 7/22/2015 22:56'!
testInvalidMoves

	|moveCheckerCommand|
"test that the checker was in the off position for player 1 and they tried to move it."
testBoard dice1 roll.
testBoard dice2 roll.
	checkerOnPoint := Checker new location: (testBoard player1Off) color:  (Color red).
	moveCheckerCommand := (MoveCheckerCommand new) checker:checkerOnPoint  location: (testBoard points at:6).

		self should:  [moveCheckerCommand execute.]
			 raise: InvalidMoveException.
			
"test that the checker was in the off position for player 2 and they tried to move it."
	checkerOnPoint := Checker new location: (testBoard player2Off) color:  (Color red).
	moveCheckerCommand := (MoveCheckerCommand new) checker:checkerOnPoint  location: (testBoard points at:6).

	

		self should:  [moveCheckerCommand execute.]
			 raise: InvalidMoveException.
	
	"test that you used one dice and the user selected one point too few."
	checkerOnPoint := Checker new location: (testBoard points at:1) color:  (Color red).
	moveCheckerCommand := (MoveCheckerCommand new) checker:checkerOnPoint  location: (testBoard points at:6).
	

	testBoard dice1: (DiceMock initialize:6).
	testBoard dice2: (DiceMock initialize:1).
	testBoard dice1 roll.
	testBoard dice2 roll.
		self should:  [moveCheckerCommand execute.]
			 raise: InvalidMoveException.! !

!MoveCheckerCommandTestCase methodsFor: 'testing' stamp: 'jChizmar 7/23/2015 16:26'!
testValidMoves
	"This tests the move of a checker from locaiton to location"
	
	| moveCheckerCommand checkerInTest|
	"Player1's checkers should add the location number to the dice to get the valid point"
	checkerInTest := testBoard player1Checkers at:1.
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (testBoard points at:7).
	testBoard player2Checkers:OrderedCollection new.
	testBoard dice1: (DiceMock initialize:6).
	testBoard dice2: (DiceMock initialize:1).
	testBoard dice1 roll.
	testBoard dice2 roll.
	moveCheckerCommand execute.
	self assert: (checkerInTest checkerLocation pointNumber = 7).

	testBoard := MainBoard initialize.
	"Player 2 rolls"
	testBoard dice1: (DiceMock initialize:2).
	testBoard dice2: (DiceMock initialize:1).
	testBoard dice1 roll.
	testBoard dice2 roll.
	
	"Player2's checkers should subtract the location number to the dice to get the valid point"
	checkerInTest := testBoard player2Checkers at:1. "Should be at location 24"
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (testBoard points at:23).
	moveCheckerCommand execute. "Execute Move 1 using Dice wtih 1"
	self assert: (checkerInTest checkerLocation pointNumber = 23).
	moveCheckerCommand := (MoveCheckerCommand new) checker:(checkerInTest) location: (testBoard points at:21).

	moveCheckerCommand execute. "Execute Move 2 using Dice wtih 6"
	self assert: (checkerInTest checkerLocation pointNumber = 21).
	"try to move a third time without rolling"
	moveCheckerCommand := (MoveCheckerCommand new)  checker:(checkerInTest)  location: (testBoard points at:8).
		self should:  [
			moveCheckerCommand execute.
		]
			 raise: InvalidMoveException.
			
	! !

TestCase subclass: #PlayerTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!PlayerTestCase methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/15/2015 20:46'!
testInitialize
	"comment stating purpose of message"

	|myPlayer|
	
	myPlayer := Player nameOfPlayer:'Jen' checkerColor:Color red.
	self assert:(myPlayer nameOfPlayer = 'Jen').! !

TestCase subclass: #RollCommandTestCase
	instanceVariableNames: 'barLocation pointLocation checkerOnPoint checkerOnBar testDice1 testDice2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!RollCommandTestCase methodsFor: 'initialize' stamp: 'jChizmar 7/17/2015 19:27'!
setUp
	"Setup the Objects for Testing"
	testDice1:=Dice new.
	testDice2:=Dice new.! !

!RollCommandTestCase methodsFor: 'initialize' stamp: 'jChizmar 7/17/2015 19:32'!
testInitialize
	"Tests the Initialization and setup of the MoveCheckerCommand"
	
	|rollCommand|
	rollCommand := (RollCommand new) dice1:testDice1  dice2:testDice2.
	rollCommand execute.
	self assert:(rollCommand dice1 currentRoll >=1 & rollCommand dice1 currentRoll <=6).
	self assert:(rollCommand dice2 currentRoll  >=1 & rollCommand dice2 currentRoll  <=6).
! !

Object subclass: #MoveCheckerCommand
	instanceVariableNames: 'checkerToMove newLocation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!MoveCheckerCommand commentStamp: 'jChizmar 7/14/2015 19:20' prior: 0!
I am a class that implements a Command called Move Checker

My main responsibility is to update the location of a checker to the location selected by the game.

I collaborate by calling methods on the Checker Class.  I take a location as input and I am called by the BGMController Class based on a user action. 

Public API and Key Messages

- execute - this method changes the location of the given checker.

  !

!MoveCheckerCommand class methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/14/2015 19:33'!
checker: checkerIn location:locationIn
	^(self new) checker: checkerIn location: locationIn! !

!MoveCheckerCommand methodsFor: 'initialize' stamp: 'jChizmar 7/14/2015 19:33'!
checker: checkerIn location:locationIn
	self checkerToMove:checkerIn.
	self newLocation:locationIn.
	^self.! !

!MoveCheckerCommand methodsFor: 'accessing' stamp: 'jChizmar 7/14/2015 19:22'!
checkerToMove
	^ checkerToMove! !

!MoveCheckerCommand methodsFor: 'accessing' stamp: 'jChizmar 7/14/2015 19:22'!
checkerToMove: anObject
	checkerToMove := anObject! !

!MoveCheckerCommand methodsFor: 'initialize' stamp: 'jChizmar 7/24/2015 22:56'!
execute
	"Execute the MoveCheckerCommand"
	| referencedBoard ruleResult| 
	referencedBoard := newLocation owningBoard.
	ruleResult := referencedBoard isMoveValid: checkerToMove toLocation:(self newLocation).

	(ruleResult rulePassed) 
	ifTrue:[
		self checkerToMove moveChecker: (self newLocation)
	]
	ifFalse:[
 		InvalidMoveException signal:  ruleResult message.
	].


	! !

!MoveCheckerCommand methodsFor: 'accessing' stamp: 'jChizmar 7/14/2015 19:22'!
newLocation
	^ newLocation! !

!MoveCheckerCommand methodsFor: 'accessing' stamp: 'jChizmar 7/14/2015 19:22'!
newLocation: anObject
	newLocation := anObject! !

Object subclass: #Observable
	instanceVariableNames: 'observers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!Observable commentStamp: 'AhmedSoliman 7/11/2015 07:00' prior: 0!
Generic Observer pattern
To Do: 
Subclass Me for any model classes that need to notify the view for changes.!

Observable subclass: #Checker
	instanceVariableNames: 'locationOnBoard color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!Checker commentStamp: 'jChizmar 7/14/2015 19:37' prior: 0!
I am the model representation of a checker on the backgammon board.

I hold the location and color of an individual checker. 

I am composed of a location (class in the Backgammon model) and a Smalltalk Color object.  

Public API and Key Messages

- color: - sets the color of the checker
- locationOnBoard: - sets the location of the checker.
- create instance by Checker new: <location> <color>

  
Internal Representation and Key Implementation Points.

    Instance Variables
	color:		<Color>
	moveChecker:		<Location>


!

!Checker class methodsFor: 'initialize' stamp: 'jChizmar 7/14/2015 19:36'!
location: locationIn color: colorIn 
	^ self new
		color: colorIn;
		moveChecker: locationIn.! !

!Checker methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 21:30'!
checkerLocation
	^ locationOnBoard! !

!Checker methodsFor: 'accessing' stamp: 'jChizmar 7/11/2015 19:29'!
color
	^ color! !

!Checker methodsFor: 'accessing' stamp: 'jChizmar 7/11/2015 19:29'!
color: colorIn
	color := colorIn! !

!Checker methodsFor: 'initialize' stamp: 'jChizmar 7/14/2015 19:36'!
location: locationIn color: colorInput
	self color:  colorInput.
	self moveChecker:  locationIn.
	^self.! !

!Checker methodsFor: 'accessing' stamp: 'jChizmar 7/14/2015 19:36'!
moveChecker: locationIn
	locationOnBoard := locationIn.
	self notifyObservers ! !

!Checker methodsFor: 'printing' stamp: 'jChizmar 7/15/2015 21:30'!
printOn: aStream
	aStream nextPutAll:'Checker Class -'; 
			  nextPutAll: (String cr);   
			  nextPutAll: 'Color:'; 
			  nextPutAll: ((self color) printString);
			  nextPutAll: (String cr); 
			  nextPutAll: 'Location:'; 
			  nextPutAll: ((self checkerLocation) printString ).! !

Observable subclass: #Dice
	instanceVariableNames: 'currentRoll usedInPlay usedInPlayDouble'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!Dice commentStamp: 'jChizmar 7/15/2015 22:05' prior: 0!
I am the class that models the rolling of one Dice. 

For the Responsibility part: I generate a random number from 1-6.

For the Collaborators Part: none

Public API and Key Messages

- roll
- initialize
!

!Dice class methodsFor: 'initialize' stamp: 'AhmedSoliman 7/25/2015 20:49'!
initialize
	"I am the initialization method for the dice."
	^self new resetFlags .	! !

!Dice methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 22:22'!
currentRoll
	^ currentRoll! !

!Dice methodsFor: 'accessing' stamp: 'AhmedSoliman 7/25/2015 20:53'!
currentRoll: anObject
	currentRoll := anObject.
! !

!Dice methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/23/2015 15:03'!
new
	Halt now.
	^self usedInPlay: false; usedInPlayDouble:false.
! !

!Dice methodsFor: 'executing' stamp: 'AhmedSoliman 7/25/2015 20:47'!
resetFlags
	self usedInPlay: false.
	self usedInPlayDouble: false! !

!Dice methodsFor: 'executing' stamp: 'AhmedSoliman 7/25/2015 21:00'!
roll
	"models the rolling of a dice"

	| newValue |
	
	newValue := 6 atRandom.
	self currentRoll: newValue.
	self resetFlags.


	self notifyObservers.
	^ newValue! !

!Dice methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 07:44'!
usedInPlay
	^ usedInPlay! !

!Dice methodsFor: 'accessing' stamp: 'jChizmar 7/23/2015 15:02'!
usedInPlay: anObject
	usedInPlay := anObject
 ! !

!Dice methodsFor: 'accessing' stamp: 'jChizmar 7/23/2015 15:01'!
usedInPlayDouble
	^ usedInPlayDouble! !

!Dice methodsFor: 'accessing' stamp: 'jChizmar 7/23/2015 15:02'!
usedInPlayDouble: anObject
	usedInPlayDouble := anObject
! !

Observable subclass: #DiceMock
	instanceVariableNames: 'currentRoll usedInPlay setValue usedInPlayDouble'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!DiceMock commentStamp: 'jChizmar 7/21/2015 17:17' prior: 0!
Mock object for testing game logic - this dice you can set up to be a specific value.!

!DiceMock class methodsFor: 'initialize' stamp: 'jChizmar 7/23/2015 15:04'!
initialize:valueIn
	"I am the initialization method for the dice."
	^self new usedInPlay:false value: valueIn; usedInPlayDouble:false.	.
	! !

!DiceMock methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 17:17'!
currentRoll
	^ currentRoll! !

!DiceMock methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 17:17'!
currentRoll: anObject
	currentRoll := anObject.
	self notifyObservers. ! !

!DiceMock methodsFor: 'initialize' stamp: 'jChizmar 7/23/2015 15:04'!
initialize:valueIn
	^self usedInPlay: false; setValue: valueIn; usedInPlayDouble:false.	.
! !

!DiceMock methodsFor: 'executing' stamp: 'jChizmar 7/21/2015 17:21'!
roll
	"Just return the static dice"
	 ^setValue.
! !

!DiceMock methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 17:18'!
setValue
	^ setValue! !

!DiceMock methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 20:03'!
setValue: anObject
	setValue := anObject.
	self currentRoll:self setValue.! !

!DiceMock methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 17:17'!
usedInPlay
	^ usedInPlay! !

!DiceMock methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 17:17'!
usedInPlay: anObject
	usedInPlay := anObject! !

!DiceMock methodsFor: 'initialize' stamp: 'jChizmar 7/21/2015 19:36'!
usedInPlay:usedIn  value:valueIn
	^self usedInPlay: usedIn; setValue: valueIn.
! !

!DiceMock methodsFor: 'accessing' stamp: 'jChizmar 7/23/2015 15:05'!
usedInPlayDouble
	^ usedInPlayDouble! !

!DiceMock methodsFor: 'accessing' stamp: 'jChizmar 7/23/2015 15:05'!
usedInPlayDouble: anObject
	usedInPlayDouble := anObject! !

Observable subclass: #Location
	instanceVariableNames: 'locationType owningBoard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!Location commentStamp: 'jChizmar 7/11/2015 20:56' prior: 0!
I am the model representation of one of the valid locations on the Backgammon board.  I am the super class of PointLocation, BarLocation, and OffLocation.   

My subclasses are referenced by the Checker to indicate what location on the board the checker is in.  


Public API and Key Messages

- 
  
Internal Representation and Key Implementation Points.

 

!

Location subclass: #BarLocation
	instanceVariableNames: 'pointNumber instance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

BarLocation class
	instanceVariableNames: 'instance'!

!BarLocation commentStamp: 'jChizmar 7/11/2015 21:08' prior: 0!
I am the model representation of the Bar on the Backgammon board. My superclass isLocation. I am a singleton.

I am referenced by the Checker to indicate that the Checker is located on the Bar.  


Public API and Key Messages

- printOn
  
Internal Representation and Key Implementation Points.

 

!

BarLocation class
	instanceVariableNames: 'instance'!

!BarLocation class methodsFor: 'initialize' stamp: 'jChizmar 7/11/2015 21:30'!
new
    instance isNil ifTrue: [instance := super new].
    ^ instance! !

!BarLocation methodsFor: 'printing' stamp: 'jChizmar 7/11/2015 20:59'!
printOn: aStream
	aStream nextPutAll:'BarLocation Class'.! !

!Location methodsFor: 'accessing' stamp: 'jChizmar 7/16/2015 18:56'!
owningBoard
	^ owningBoard! !

!Location methodsFor: 'accessing' stamp: 'jChizmar 7/16/2015 18:56'!
owningBoard: anObject
	owningBoard := anObject! !

!Location methodsFor: 'printing' stamp: 'jChizmar 7/11/2015 21:00'!
printOn: aStream
	aStream nextPutAll:'Location Class'.! !

Location subclass: #OffLocation
	instanceVariableNames: 'player numCheckersOnHome'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

OffLocation class
	instanceVariableNames: 'instance'!

!OffLocation commentStamp: 'jChizmar 7/11/2015 21:09' prior: 0!
I am the model representation of being off the board in Backgammon. My superclass isLocation. 

I am referenced by the Checker to indicate that the Checker is located on the  board.  


Public API and Key Messages

- player - gets the player that is associated with the off location
- player :- sets the player that is associated with the off location
- printOn
  
Internal Representation and Key Implementation Points.

 

!

OffLocation class
	instanceVariableNames: 'instance'!

!OffLocation class methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/23/2015 20:48'!
player: playerIn
	^ self new playerInit: playerIn.! !

!OffLocation methodsFor: 'accessing' stamp: 'xl 7/24/2015 00:44'!
incrementNumCheckersOnHome
	self numCheckersOnHome:(self numCheckersOnHome) + 1.
	self notifyObservers.
	self numCheckersOnHome = 15
		ifTrue: [ 
			MorphicUIManager default alert: 'You Win!!' title: 'Game Over'
		].! !

!OffLocation methodsFor: 'as yet unclassified' stamp: 'xl 7/23/2015 23:08'!
numCheckersOnHome
	^numCheckersOnHome! !

!OffLocation methodsFor: 'accessing' stamp: 'xl 7/23/2015 22:53'!
numCheckersOnHome: anObject
	numCheckersOnHome := anObject.
	self notifyObservers. ! !

!OffLocation methodsFor: 'accessing' stamp: 'jChizmar 7/23/2015 20:52'!
owningBoard: bIn player: playerIn
	owningBoard := bIn.
	player := playerIn.! !

!OffLocation methodsFor: 'accessing' stamp: 'xl 7/23/2015 22:54'!
player
	^ player! !

!OffLocation methodsFor: 'accessing' stamp: 'xl 7/24/2015 00:33'!
player: aPlayer
	player := aPlayer! !

!OffLocation methodsFor: 'initialize' stamp: 'jChizmar 7/23/2015 20:47'!
playerInit: playerIn
	self player:playerIn.
	self numCheckersOnHome:0.
	^self.! !

!OffLocation methodsFor: 'printing' stamp: 'jChizmar 7/13/2015 20:50'!
printOn:aStream 
	aStream nextPutAll:'Off Location Class -'; 
			  nextPutAll: (String cr); 
			  nextPutAll: 'Player:'; 
			  nextPutAll: ((self player) printString).! !

Location subclass: #PointLocation
	instanceVariableNames: 'pointNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!PointLocation commentStamp: 'jChizmar 7/11/2015 21:09' prior: 0!
I am the model representation of one the Point locations on the Backgammon board. My superclass is location

I am referenced by the Checker to indicate that the Checker is located on a specific Point on the gameboard.


Public API and Key Messages

- pointNumber - gets the number of the point that is associated with the off location
- pointNumber :- sets the number of the point that is associated with the pointlocation
- printOn
  
Internal Representation and Key Implementation Points.

 

!

!PointLocation class methodsFor: 'initialize' stamp: 'jChizmar 7/16/2015 19:44'!
number: pointNumberIn
	^ self new
		pointNumber:pointNumberIn.! !

!PointLocation methodsFor: 'initialize' stamp: 'jChizmar 7/16/2015 19:43'!
number: pointNumberIn.
	self pointNumber:pointNumberIn.
	^self.! !

!PointLocation methodsFor: 'accessing' stamp: 'jChizmar 7/11/2015 21:06'!
pointNumber
	^pointNumber.! !

!PointLocation methodsFor: 'accessing' stamp: 'jChizmar 7/11/2015 21:01'!
pointNumber: pointNumberIn
	pointNumber := pointNumberIn.! !

!PointLocation methodsFor: 'printing' stamp: 'jChizmar 7/13/2015 20:55'!
printOn:aStream 
	aStream nextPutAll:'PointLocation Class -'; 
			  nextPutAll: (String cr);  
			  nextPutAll: 'PointNumber:'; 
			  nextPutAll: ((self pointNumber) printString).! !

!Observable methodsFor: 'as yet unclassified' stamp: 'AhmedSoliman 7/13/2015 15:13'!
addObserver: anObserver
    observers add: anObserver.
	anObserver observable: self! !

!Observable methodsFor: 'as yet unclassified' stamp: 'AhmedSoliman 7/11/2015 06:57'!
initialize
    observers := OrderedCollection new.! !

!Observable methodsFor: 'as yet unclassified' stamp: 'DavidOvergaard 7/21/2015 21:21'!
notifyObservers
    observers do: [ :each | each notify ].! !

!Observable methodsFor: 'as yet unclassified' stamp: 'AhmedSoliman 7/11/2015 06:58'!
removeObserver: anObserver
    observers remove: anObserver.! !

Object subclass: #Observer
	instanceVariableNames: 'observable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!Observer commentStamp: 'AhmedSoliman 7/11/2015 07:01' prior: 0!
An Observer for changes in the observable class!

Observer subclass: #CheckerObserver
	instanceVariableNames: 'locationNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!CheckerObserver commentStamp: 'DavidOvergaard 7/16/2015 22:06' prior: 0!
Observe a checker and update the view!

!CheckerObserver methodsFor: 'as yet unclassified' stamp: 'DavidOvergaard 7/16/2015 22:07'!
notify
	"method to be called when the checker class changed."
	Transcript show:'The Checker I am watching changed!!'.
	Transcript show: String cr.
	Transcript show:observable printString.
	self locationNumber:(observable checkerLocation pointNumber ).! !

Observer subclass: #CheckerView
	instanceVariableNames: 'circle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!CheckerView commentStamp: '<historical>' prior: 0!
CheckerView!

!CheckerView methodsFor: 'as yet unclassified' stamp: 'xl 7/15/2015 19:06'!
initialize
	super initialize.
	circle := (CircleMorph new) 
					color: (observable color); 
					borderWidth: 0; 
					left:20;
					top: 20;
					extent:100 @ 100.! !

Observer subclass: #MainBoard
	instanceVariableNames: 'player1 player2 player1Checkers player2Checkers points bar player1Off player2Off dice1 dice2 status player1BOChecker player2BOChecker offLocationP1 offLocationP2 itsMyTurn'
	classVariableNames: 'instance'
	poolDictionaries: ''
	category: 'BackgammonModel'!

!MainBoard commentStamp: 'AhmedSoliman 7/13/2015 14:52' prior: 0!
This is the mainboard of the backgammon game!

!MainBoard class methodsFor: 'initialize' stamp: 'AhmedSoliman 7/25/2015 21:28'!
initialize
	| mySelf i |
	instance isNil ifFalse: [^instance ].
    
	mySelf := self new.	"initialize"	"Create Players"
	mySelf player1: (Player new name: 'Jen' checkerColor: Color red).
	mySelf player2: (Player new name: 'Karen' checkerColor: Color black).	
	mySelf itsMyTurn:	mySelf player2.
		"Create Board Locations"	"Create Bar "
			
	mySelf bar: ((BarLocation new) owningBoard: mySelf).	"Create Points"
	i := 1.
	mySelf points: OrderedCollection new.
	[ i < 25 ]
		whileTrue: [ 
			mySelf points add: (((PointLocation new number: i)) owningBoard: mySelf).
			i := i + 1 ].	
	
	"Initialize the Checkers to the Start State"
	mySelf player1Checkers: OrderedCollection new.
	mySelf player2Checkers: OrderedCollection new.
	
	MainBoard initializePlayer1Checkers:mySelf checkerColor:Color red.
	MainBoard initializePlayer2Checkers:mySelf checkerColor:Color black.
	
	mySelf player1BOChecker: (NumberCheckerView diameter:50 color: (Color red)).
	mySelf player2BOChecker: (NumberCheckerView diameter:50 color: (Color black)).
	
	
	"MainBoard initializeBearingOffPlayer1Checkers:mySelf checkerColor:Color red.
	MainBoard initializeBearingOffPlayer2Checkers:mySelf checkerColor:Color black."
	
	
	"Initialize the Off Location"
	mySelf player1Off: (((OffLocation new owningBoard: mySelf player: (mySelf player1)) numCheckersOnHome:0)).
	mySelf player2Off: (((OffLocation new owningBoard: mySelf player: (mySelf player2)) numCheckersOnHome:0)).
	mySelf player1Off owningBoard: mySelf.
	mySelf player2Off owningBoard: mySelf.
	mySelf offLocationP1: (mySelf player1Off).
	mySelf offLocationP2: (mySelf player2Off).
	
	"Initialize the Dice"
	mySelf dice1:Dice initialize.
	mySelf dice2:Dice initialize.
	"mySelf dice1 roll.
	mySelf dice2 roll."
	"mySelf runMoveValidation:true."
	
	(mySelf dice1) addObserver: mySelf.
	mySelf status: 0.
	^instance := mySelf.
	! !

!MainBoard class methodsFor: 'initialize' stamp: 'jChizmar 7/24/2015 23:03'!
initializeBearingOffPlayer1Checkers: selfReference checkerColor: colorIn
	"Initialize Player 1 Checkers"
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 19) color: colorIn).	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 20) color: colorIn).	
		
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 20) color: colorIn).	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 20) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 20) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 21) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 21) color: colorIn).
	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 21) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 22) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 22) color: colorIn).
	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 23) color: colorIn).	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 23) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 24) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 24) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 24) color: colorIn).! !

!MainBoard class methodsFor: 'initialize' stamp: 'jChizmar 7/23/2015 19:14'!
initializeBearingOffPlayer2Checkers: selfReference checkerColor: colorIn
	"Initialize Player 2 Checkers"
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 1) color: colorIn).	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 1) color: colorIn).	
		
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 2) color: colorIn).	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 2) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 3) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 3) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 4) color: colorIn).
	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 4) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 5) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 5) color: colorIn).
	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 5) color: colorIn).	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).! !

!MainBoard class methodsFor: 'initialize' stamp: 'jChizmar 7/21/2015 07:55'!
initializePlayer1Checkers: selfReference checkerColor: colorIn
	"Initialize Player 1 Checkers"
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 1) color: colorIn).	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 1) color: colorIn).	
		
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 12) color: colorIn).	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 12) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 12) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 12) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 12) color: colorIn).
	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 17) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 17) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 17) color: colorIn).
	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 19) color: colorIn).	
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 19) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 19) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 19) color: colorIn).
	selfReference player1Checkers add: (Checker new location: (selfReference points at: 19) color: colorIn).! !

!MainBoard class methodsFor: 'initialize' stamp: 'jChizmar 7/20/2015 21:52'!
initializePlayer2Checkers: selfReference checkerColor: colorIn
	"Initialize Player 2 Checkers"
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 24) color: colorIn).	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 24) color: colorIn).	
		
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 13) color: colorIn).	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 13) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 13) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 13) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 13) color: colorIn).
	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 8) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 8) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 8) color: colorIn).
	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).	
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).
	selfReference player2Checkers add: (Checker new location: (selfReference points at: 6) color: colorIn).! !

!MainBoard class methodsFor: 'initialize' stamp: 'AhmedSoliman 7/25/2015 21:21'!
newGame
 	instance := nil. 
	^MainBoard initialize.! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
bar
	^ bar! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
bar: anObject
	bar := anObject! !

!MainBoard methodsFor: 'rules' stamp: 'jChizmar 7/24/2015 23:37'!
canBearOff:numberIn
	"This method returns true if the player can bear off"
|  numCheckerInHome myOffLocation checkerWithOutOffs|
numCheckerInHome := 0.
(numberIn= 1)
ifTrue:[
		checkerWithOutOffs := self player1Checkers select:[:checkerOnList | ((checkerOnList checkerLocation) = player1Off)not].
		numCheckerInHome :=(checkerWithOutOffs select:[:checkerOnList1 | ((checkerOnList1 checkerLocation pointNumber >=19) & (checkerOnList1 checkerLocation pointNumber <=24))]) size. 
		myOffLocation := player1Off.
	]
ifFalse:[
		checkerWithOutOffs := self player2Checkers select:[:checkerOnList | ((checkerOnList checkerLocation) = player2Off)not].
		numCheckerInHome :=(checkerWithOutOffs select:[:checkerOnList | ((checkerOnList checkerLocation pointNumber >=1) & (checkerOnList checkerLocation pointNumber <=6))]) size.
		myOffLocation := player2Off.
	].
^ ((numCheckerInHome + (myOffLocation numCheckersOnHome)) =15)
	! !

!MainBoard methodsFor: 'rules' stamp: 'jChizmar 7/24/2015 19:20'!
checkForInvalidMoves:checkerIn toLocation:locationIn playerNum:playerIn

|returnValue|
returnValue := RuleOutput initialize.
	(self checkForOppoentsCheckers:locationIn playerNum:playerIn) 
		ifFalse:[
			returnValue rulePassed: false;
			message:'Opponents Checkers are at this location - this iteration does not allow hitting'.
		]
		ifTrue:[
			"If the checker is on the bar - always invalid for this iteration"
			(checkerIn checkerLocation = self bar | locationIn = self bar)
				ifTrue:[
					returnValue rulePassed:false;
					message:'Temporary Fail - this iteration Bar Processing not Complete.'.
				]
				ifFalse:[
					"If the checker is on the off location - always Invalid"
					((checkerIn checkerLocation = self player1Off) | (checkerIn checkerLocation = self player2Off))
						ifTrue:[
							returnValue rulePassed:false;
							message:'Fail - Attempting to move checkers off of either Player1 or Player2 off location'.
						]
						
						ifFalse:[
						
							"find out if the checker being moved belongs to the player that whose turn it is."
							(self itsMyTurn checkerColor = checkerIn color)
							ifFalse:[
								returnValue rulePassed:false;
								message:'Fail - Player attempted to move a checker of another player'.
								].	
						].
				].
			].
^returnValue.! !

!MainBoard methodsFor: 'rules' stamp: 'jChizmar 7/23/2015 16:25'!
checkForOppoentsCheckers:locationIn playerNum:playerIn
|oppCheckerNum|
oppCheckerNum :=0.

(playerIn = 1)
ifTrue:[
		oppCheckerNum :=(self player2Checkers select:[:checkerOnList | (checkerOnList checkerLocation) = locationIn]) size.
	]
ifFalse:[
		oppCheckerNum :=(self player1Checkers select:[:checkerOnList | (checkerOnList checkerLocation) = locationIn]) size.
	].
^ (oppCheckerNum =0)! !

!MainBoard methodsFor: 'gameMechanics' stamp: 'jChizmar 7/16/2015 20:31'!
checkersOnLocation: locationIn.
	"I return the collection of checkers fthat are currently in play for a given location - consider both Player 1 and Player 2's checkers.  
	This method does not return checkers that are in the 'off' Locations.  If an 'off' location is passed in, the method will return an empty list.  "

	 ^ ((self player1Checkers select:[:checkerOnList | (checkerOnList checkerLocation) = locationIn]), 
		(self player2Checkers select:[:checkerOnList | (checkerOnList checkerLocation) = locationIn])).! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/17/2015 19:15'!
dice1
	^ dice1! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/17/2015 19:15'!
dice1: anObject
	dice1 := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/17/2015 19:15'!
dice2
	^ dice2! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/17/2015 19:15'!
dice2: anObject
	dice2 := anObject! !

!MainBoard methodsFor: 'rules' stamp: 'jChizmar 7/24/2015 23:59'!
hasCheckersOnGreater: locationNum diceValue:dieIn playerNum:playerIn
	"special processing for Bear Off - can I use dice Higher/Lower than my point location>"
	||

^ false.! !

!MainBoard methodsFor: 'rules' stamp: 'jChizmar 7/23/2015 15:48'!
isDiceUsed:diceIn hasDoubles:isADouble
	"Determine if a Die is fully used."
	|isFullyUsed|
	isFullyUsed := false.
	(isADouble)
	ifTrue:[
		isFullyUsed := diceIn usedInPlay & diceIn usedInPlayDouble.
		]
	ifFalse:[
			isFullyUsed := diceIn usedInPlay.
		].
	^isFullyUsed.! !

!MainBoard methodsFor: 'gameMechanics' stamp: 'jChizmar 7/23/2015 15:37'!
isDoubles
	"comment stating purpose of message"

	^(dice1 currentRoll = dice2 currentRoll)! !

!MainBoard methodsFor: 'rules' stamp: 'jChizmar 7/25/2015 00:07'!
isMoveValid: checkerIn toLocation:locationIn
	"I determine if the requested Checker Move is valid"
	|returnValue multNeg1 dice1UsedInThisRoll playerRolledDoubles dice1FullyUsed dice2FullyUsed playerNumber playerCanBearOff justBeyondLastPoint moveToNumber checkersGreaterMyLocation |
	returnValue := RuleOutput initialize.
	
	dice1UsedInThisRoll := false.
	playerCanBearOff := false.
	justBeyondLastPoint :=25.
	
	"figure out if this is a player2  checker"
	multNeg1:=1.
	playerNumber :=1.

	((self player2Checkers select:[:checkerOnList | checkerOnList = checkerIn]) size >= 1)
		ifTrue:[
				multNeg1 :=-1.
				playerNumber :=2.
				justBeyondLastPoint := 0.
			].
		
	"Figure out of the player rolled doubles"
	playerRolledDoubles := self isDoubles.
	dice1FullyUsed :=  self isDiceUsed:dice1 hasDoubles:playerRolledDoubles.
	dice2FullyUsed :=  self isDiceUsed:dice2 hasDoubles:playerRolledDoubles.

	
	(dice1FullyUsed & dice2FullyUsed) 
			ifTrue:[
				returnValue rulePassed: false;
				message:'Both Dice have been used in Play - no legal move'.
			]
			ifFalse:[
		returnValue := self checkForInvalidMoves:checkerIn toLocation:locationIn playerNum: playerNumber.
		(returnValue rulePassed)
		ifTrue:[
		"If all of the basic invalid moves have been ruled out, check for compliance with the dice. "
		
		"figure out if this player can bear off"

		playerCanBearOff := self canBearOff:playerNumber.
		((playerCanBearOff = false) &  (self isMyOffLocation:locationIn playerNum:playerNumber))
		ifTrue:[
				returnValue rulePassed: false;
				message:'Player Cannot Bear off, but has tried to move to the HomeLocation'.
			]
		ifFalse:[
			(playerCanBearOff &  (self isMyOffLocation:locationIn playerNum:playerNumber))
			ifTrue:[
				"---------------------------------This is Bear off Logic -------------------------------------------"
				
				(dice1FullyUsed) 
				ifFalse:[ 
					moveToNumber := checkerIn checkerLocation pointNumber + (dice1 currentRoll*multNeg1).
					checkersGreaterMyLocation :=  self hasCheckersOnGreater:(checkerIn checkerLocation pointNumber) diceValue:(dice1 currentRoll) playerNum:playerNumber.
					(moveToNumber = justBeyondLastPoint) 
					ifTrue:[
							self setUsedIndicatorForDice:dice1 hasDoubles:playerRolledDoubles. 
							dice1UsedInThisRoll := true.				
					]
				].								
		
				(dice1UsedInThisRoll = false) "Thus check Dice 2"
				ifTrue:[
				(dice2FullyUsed = false) 
						ifTrue:[
							moveToNumber := checkerIn checkerLocation pointNumber + (dice2 currentRoll*multNeg1).
							checkersGreaterMyLocation :=  self hasCheckersOnGreater:(checkerIn checkerLocation pointNumber) diceValue:(dice2 currentRoll) playerNum:playerNumber.
							(moveToNumber = justBeyondLastPoint) 
							ifTrue:[
								self setUsedIndicatorForDice:dice2 hasDoubles:playerRolledDoubles.
							]
							ifFalse:[
								returnValue rulePassed: false;
								message:'Player selected an invalid point for the dice roll'.
											
							].			
											
						].
								
			].
				
				"Increment the off location's count because it is moved off"
				locationIn incrementNumCheckersOnHome.
				"-------------------------------End Bear Logic---------------------------------------------"
			]
		ifFalse:[
			"determine if it is a legal move - it is moving from point to point"
			(dice1FullyUsed) 
			ifFalse:[ 
					(checkerIn checkerLocation pointNumber + (dice1 currentRoll*multNeg1) = locationIn pointNumber) 
									ifTrue:[
										self setUsedIndicatorForDice:dice1 hasDoubles:playerRolledDoubles. 
										dice1UsedInThisRoll := true.				
									]
			].								
		
			(dice1UsedInThisRoll = false) "Thus check Dice 2"
			ifTrue:[
				(dice2FullyUsed = false) 
						ifTrue:[
							(checkerIn checkerLocation pointNumber + (dice2 currentRoll*multNeg1) = locationIn pointNumber) 
							ifTrue:[
								self setUsedIndicatorForDice:dice2 hasDoubles:playerRolledDoubles.
							]
							ifFalse:[
								returnValue rulePassed: false;
								message:'Player selected an invalid point for the dice roll'.
											
							].			
											
						].
								
			].
		].
	].
	].
].
	
^returnValue.       ! !

!MainBoard methodsFor: 'gameMechanics' stamp: 'jChizmar 7/23/2015 20:24'!
isMyOffLocation:locationIn playerNum:playerNumIn
	"returns true if the location in is the player's off location"
|output|
(playerNumIn= 1 & (locationIn = self player1Off ))
ifTrue:[
		output :=true.
	]
ifFalse:[
		
		(playerNumIn= 2 & (locationIn = self player2Off ))
			ifTrue:[
				output :=true.
			]	
			ifFalse:[
				output :=false.
			].
	] .
^output.! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/24/2015 19:05'!
itsMyTurn
	^ itsMyTurn! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/24/2015 19:05'!
itsMyTurn: anObject
	itsMyTurn := anObject! !

!MainBoard methodsFor: 'gameMechanics' stamp: 'AhmedSoliman 7/25/2015 19:59'!
notify
	"Figure out if the player's turn is over because the dice has been rolled"

	(self itsMyTurn = self player1)
	ifTrue:[self itsMyTurn: self player2. self status: 2]
	ifFalse:[self itsMyTurn: self player1. self status: 1] .
	

! !

!MainBoard methodsFor: 'accessing' stamp: 'xl 7/24/2015 00:24'!
offLocationP1
	^offLocationP1! !

!MainBoard methodsFor: 'accessing' stamp: 'xl 7/24/2015 00:24'!
offLocationP1: anObject
	offLocationP1 := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'xl 7/24/2015 00:24'!
offLocationP2
	^offLocationP2! !

!MainBoard methodsFor: 'accessing' stamp: 'xl 7/24/2015 00:24'!
offLocationP2: anObject
	offLocationP2 := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
player1
	^ player1! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:55'!
player1: anObject
	player1 := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'xl 7/23/2015 23:22'!
player1BOChecker
	^ player1BOChecker! !

!MainBoard methodsFor: 'accessing' stamp: 'xl 7/23/2015 23:22'!
player1BOChecker: anObject
	player1BOChecker := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
player1Checkers
	^ player1Checkers! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:55'!
player1Checkers: anObject
	player1Checkers := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
player1Off
	^ player1Off! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:55'!
player1Off: anObject
	player1Off := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
player2
	^ player2! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:55'!
player2: anObject
	player2 := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'xl 7/23/2015 23:22'!
player2BOChecker
	^ player2BOChecker! !

!MainBoard methodsFor: 'accessing' stamp: 'xl 7/23/2015 23:22'!
player2BOChecker: anObject
	player2BOChecker := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
player2Checkers
	^ player2Checkers! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:55'!
player2Checkers: anObject
	player2Checkers := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
player2Off
	^ player2Off! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:55'!
player2Off: anObject
	player2Off := anObject! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 19:58'!
points
	^ points! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:55'!
points: anObject
	points := anObject! !

!MainBoard methodsFor: 'gameMechanics' stamp: 'jChizmar 7/21/2015 11:36'!
resetBoardForPlayerTurn
	"This method resets the board when the players change truns"

	self dice1: Dice initialize.
	self dice2: Dice initialize.! !

!MainBoard methodsFor: 'gameMechanics' stamp: 'jChizmar 7/23/2015 16:06'!
setUsedIndicatorForDice:diceIn hasDoubles: isADouble
	"Set the Used Indicators Correctly"
	|isFullyUsed|
	isFullyUsed := false.
	
	(isADouble)
	ifFalse:[
		diceIn usedInPlay:true.
		]
	ifTrue:[
		(diceIn usedInPlay)
			ifTrue:[
					diceIn usedInPlayDouble:true.
				]
			ifFalse:[
					diceIn usedInPlay:true.
				].
		].
	
! !

!MainBoard methodsFor: 'accessing' stamp: 'jChizmar 7/20/2015 22:33'!
status
	^ status! !

!MainBoard methodsFor: 'accessing' stamp: 'AhmedSoliman 7/25/2015 21:23'!
status: anObject

	status := anObject! !

Observer subclass: #MockView
	instanceVariableNames: 'locationNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!MockView commentStamp: 'jChizmar 7/15/2015 23:11' prior: 0!
I am a class the tests the Model's Observer Pattern for Checker and Dice.



For the Collaborators Part: I observe the Checkers and Dice classes.
Public API and Key Messages

- CheckerChanged   
- DiceChanged
!

!MockView class methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/15/2015 23:30'!
checker:checkerIn	
	^ self new checker: checkerIn.! !

!MockView methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 23:42'!
locationNumber
	^ locationNumber! !

!MockView methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 23:42'!
locationNumber: anObject
	locationNumber := anObject! !

!MockView methodsFor: 'initialize' stamp: 'jChizmar 7/20/2015 22:18'!
notify:aString
	"method to be called when the checker class changed."
	Transcript show:'The Checker I am watching changed!!'.
	Transcript show:observable printString.
	self locationNumber:(observable checkerLocation pointNumber ).

! !

!Observer methodsFor: 'as yet unclassified' stamp: 'AhmedSoliman 7/11/2015 07:03'!
notify
    "Action to notify on e.g.: Change in the checker location"
    Transcript show: 'Some things changed in ', observable asString.! !

!Observer methodsFor: 'as yet unclassified' stamp: 'AhmedSoliman 7/11/2015 07:01'!
observable
    ^ observable.! !

!Observer methodsFor: 'as yet unclassified' stamp: 'AhmedSoliman 7/11/2015 07:01'!
observable: anObject
    observable := anObject.! !

Object subclass: #Player
	instanceVariableNames: 'playerName checkerColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!Player commentStamp: 'AhmedSoliman 7/13/2015 14:52' prior: 0!
I represent the backgammon players of the game!

!Player class methodsFor: 'initialize' stamp: 'jChizmar 7/15/2015 20:32'!
createCheckers


	| i myCheckers |
	i:=0.
	myCheckers := Array new.
	(i<14) whileTrue:
	[ 
		myCheckers at: i put: Checker new.
		i := i+1.
	 ]! !

!Player class methodsFor: 'initialize' stamp: 'jChizmar 7/15/2015 20:36'!
name: nameIn	 checkerColor: colorIn.
	^ self new
		name:nameIn;
		checkerColor:colorIn.! !

!Player class methodsFor: 'initialize' stamp: 'jChizmar 7/15/2015 20:39'!
nameOfPlayer: nameIn	 checkerColor: colorIn.
	^ self new
		nameOfPlayer:nameIn;
		checkerColor:colorIn.! !

!Player methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:12'!
checkerColor
	^ checkerColor! !

!Player methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:12'!
checkerColor: anObject
	checkerColor := anObject! !

!Player methodsFor: 'initialize' stamp: 'jChizmar 7/15/2015 20:39'!
name: nameIn checkerColor:colorIn
	"comment stating purpose of message"

	self nameOfPlayer: nameIn.
	self checkerColor:colorIn.
	^self.! !

!Player methodsFor: 'accessing' stamp: 'jChizmar 7/15/2015 20:43'!
nameOfPlayer
	^ playerName.! !

!Player methodsFor: 'initialize' stamp: 'jChizmar 7/15/2015 20:45'!
nameOfPlayer: nameIn
	"comment stating purpose of message"

	playerName := nameIn.
.! !

!Player methodsFor: 'initialize' stamp: 'jChizmar 7/15/2015 20:40'!
nameofPlayer: nameIn checkerColor:colorIn
	"comment stating purpose of message"

	self nameOfPlayer: nameIn.
	self checkerColor:colorIn.
	^self.! !

!Player methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/11/2015 21:08'!
printOn: aStream
	aStream nextPutAll:'PlayerClass'.! !

Object subclass: #RollCommand
	instanceVariableNames: 'dice1 dice2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!RollCommand commentStamp: 'jChizmar 7/17/2015 19:07' prior: 0!
This Command rolls two dice and returns the values.!

!RollCommand class methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/17/2015 19:28'!
dice1:dice1In dice2:dice2In
	^(self new) dice1:dice1In dice2:dice2In.! !

!RollCommand methodsFor: 'accessing' stamp: 'jChizmar 7/17/2015 19:09'!
dice1
	^ dice1! !

!RollCommand methodsFor: 'accessing' stamp: 'jChizmar 7/17/2015 19:09'!
dice1: anObject
	dice1 := anObject! !

!RollCommand methodsFor: 'initialize' stamp: 'jChizmar 7/17/2015 19:14'!
dice1: dice1In dice2:dice2In
	self dice1:dice1In.
	self dice2:dice2In.
	^self.! !

!RollCommand methodsFor: 'accessing' stamp: 'jChizmar 7/17/2015 19:09'!
dice2
	^ dice2! !

!RollCommand methodsFor: 'accessing' stamp: 'jChizmar 7/17/2015 19:09'!
dice2: anObject
	dice2 := anObject! !

!RollCommand methodsFor: 'initialize' stamp: 'jChizmar 7/17/2015 19:12'!
execute
	"Execute the Roll Command"
	dice1 roll.
	dice2 roll.
	! !

Object subclass: #RuleOutput
	instanceVariableNames: 'rulePassed message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!RuleOutput commentStamp: 'jChizmar 7/21/2015 11:41' prior: 0!
I am a class that returns whether the rule passed.  If the rule did not pass, it also returns the reason for debugging. !

!RuleOutput class methodsFor: 'as yet unclassified' stamp: 'jChizmar 7/21/2015 17:11'!
initialize
	"comment stating purpose of message"

	^self new messageIn: '' passed:true.! !

!RuleOutput methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 11:42'!
message
	^ message! !

!RuleOutput methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 11:42'!
message: anObject
	message := anObject! !

!RuleOutput methodsFor: 'initialize' stamp: 'jChizmar 7/21/2015 11:44'!
messageIn:msgIn passed:passedIn
	"comment stating purpose of message"

	self message:msgIn.
	self rulePassed:passedIn.
	^self.
! !

!RuleOutput methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 11:42'!
rulePassed
	^ rulePassed! !

!RuleOutput methodsFor: 'accessing' stamp: 'jChizmar 7/21/2015 11:42'!
rulePassed: anObject
	rulePassed := anObject! !

Error subclass: #InvalidMoveException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BackgammonModel'!

!InvalidMoveException commentStamp: 'jChizmar 7/21/2015 11:53' prior: 0!
This class implement the error when the user moves to an invalid position in the game rules!
Dice initialize!
RuleOutput initialize!
MainBoard initialize!
